library attribute_directives.shorten_overflow_text;

import 'dart:html' as html;
import 'dart:math';
import 'package:angular2/angular2.dart';

@Directive(selector: '[foShortenOverflow]')
class ShortenOverflowTextDirective implements AfterContentChecked
{
  ShortenOverflowTextDirective(ElementRef element_ref)
      : _element = element_ref.nativeElement, _parentElement = element_ref.nativeElement.parent
  {
    html.window.onResize.listen((_) => _adjustWidth());
  }

  void ngAfterContentChecked()
  {
    _adjustWidth();
  }

  void _adjustWidth()
  {
    if (_fullText == null) _fullText = _element.text;
    _element.text = _fullText;

    html.Rectangle<double> parentRect = _parentElement.getBoundingClientRect();
    html.Rectangle<double> rect = _element.getBoundingClientRect();
    double parentWidth = parentRect.right - parentRect.left;
    double width = rect.right - rect.left;
    double dotMargin = 20.0;
    int overflowGuard = 999;

    String croppedValue = _fullText;

    while (width + dotMargin > parentWidth && overflowGuard > 0)
    {
      croppedValue = croppedValue.substring(0, max(croppedValue.length - 1, 0));
      _element.text = croppedValue;
      rect = _element.getBoundingClientRect();
      width = rect.right - rect.left;
      overflowGuard--;
    }
    if (overflowGuard < 999) _element.text += "...";
  }

  @Input('foShortenOverflow')
  void set text(String value)
  {
    _fullText = value;
    _adjustWidth();
  }

  final html.Element _element;
  final html.Element _parentElement;
  String _fullText;
}